# -*- coding: UTF-8 -*-

"""
indicator - 指标计算模块
====================================================================
"""

from .share import ShareDayIndicator
from .market import MarketIndicator
from .meta import check_indicator_meta




# coding: utf-8

"""
Account - 虚拟账户，用于仿真交易

仿真交易分为两种模式：
1）strict - 严格模式，只能在正常交易时间段进行虚拟交易，参照真实交易规则；
2）loose - 宽松模式，不进行任何限制，允许任意添加交易记录
====================================================================
"""

import os
from datetime import datetime
import json

from tma import ACCOUNT_PATH
from tma.collector import get_price


class Order(object):
    """订单对象"""
    pass


class Account:
    def __init__(self, name, fund=-1, mode="strict"):
        self.name = name
        self.mode = mode
        self.fund = fund
        self.path = os.path.join(ACCOUNT_PATH, "account_%s.json" % self.name)
        self._read_info()

    def _read_info(self):
        if os.path.exists(self.path):
            self.info = json.load(open(self.path, 'r'))
        else:
            self.info = {
                "name": self.name,
                "path": self.path,
                "fund": self.fund,  # 把fund设置为-1表示不限制资金大小
                "trades": {},
                "create_date": datetime.now().date().__str__(),
            }

    def _save_info(self):
        json.dump(self.info, open(self.path, 'w'), indent=2)

    def buy(self, code, amount, price=None):
        if not price:
            price = get_price(code)
        record = {
            "code": code,
            "amount": amount,
            "price": price,
            "date": datetime.now().date().__str__(),
            "kind": "BUY"
        }
        if code not in self.info['trades'].keys():
            self.info['trades'][code] = []
        self.info['trades'][code].append(record)
        self._save_info()

    def sell(self):
        pass








# -*- coding: utf-8 -*-
"""
sms - 预警消息推送模块
====================================================================
"""

from retrying import retry
import requests
import os
import smtplib
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from tma import SCKEY


@retry(stop_max_attempt_number=6)
def server_chan_push(title, content, sckey=None):
    """使用server酱推送消息到微信，关于server酱，
    请参考：http://sc.ftqq.com/3.version

    :param title: str
        消息标题
    :param content: str
        消息内容，最长64Kb，可空，支持MarkDown
    :param sckey: str
        从[Server酱](https://sc.ftqq.com/3.version)获取的key
    :return: None
    """
    if not sckey and not SCKEY:
        raise ValueError("请配置SCKEY，如果还没有SCKEY，"
                         "可以到这里申请一个：http://sc.ftqq.com/3.version")

    # 优先使用系统内部配置的SCKEY
    sckey = SCKEY if SCKEY else sckey
    url = 'https://sc.ftqq.com/%s.send' % sckey
    requests.post(url, data={'text': title, 'desp': content})


class EmailSender:
    """
    example
    ----------------
    subject = '附件发送测试'
    content = '检测附件发送是否成功'
    files = ['test.py',
             'f:\\已下载.txt']

    e = EmailSender('zeng_bin8888@163.com', pw=pw, service='163')
    e.send_email(to, subject, content, files)
    """

    def __init__(self, from_, pw, service='163'):
        self.from_ = from_  # 用于发送email的邮箱
        self.pw = pw  # 发送email的邮箱密码

        # 登录邮箱
        self.smtp = smtplib.SMTP()
        smtps = {
            "qq": 'smtp.exmail.qq.com',
            "163": 'smtp.163.com'
        }
        assert service in smtps.keys(), "目前仅支持：%s" % str(smtps.keys()).strip("dict_keys()")
        smtp = smtps[service]
        self.smtp.connect(smtp)
        self.smtp.login(self.from_, self.pw)

    def construct_msg(self, to, subject, content, files=None):
        """构造email信息

        parameters
        ---------------
        subject     邮件主题
        content     邮件文本内容
        files       附件（list）,可以是相对路径下的文件，也可以是绝对路径下的文件

        return
        --------------
        msg         构造好的邮件信息
        """
        msg = MIMEMultipart()
        msg['from'] = self.from_
        msg['to'] = to
        msg['subject'] = subject
        txt = MIMEText(content)
        msg.attach(txt)

        # 添加附件
        if files is not None:
            for file in files:
                f = MIMEApplication(open(file, 'rb').read())
                f.add_header('Content-Disposition', 'attachment',
                             filename=os.path.split(file)[-1])
                msg.attach(f)

        return msg

    @retry(stop_max_attempt_number=6)
    def send_email(self, to, subject, content, files=None):
        """登录邮箱，发送msg到指定联系人"""
        smtp = self.smtp
        msg = EmailSender.construct_msg(self, to, subject, content, files=files)
        smtp.sendmail(self.from_, to, str(msg))

    def quit(self):
        smtp = self.smtp
        smtp.quit()  # 退出登录


def send_email(from_, pw, to, subject, content, files=None, service='163'):
    """邮件发送（支持附件），推荐使用163邮箱"""
    se = EmailSender(from_=from_, pw=pw, service=service)
    se.send_email(to=to, subject=subject, content=content, files=files)
    se.quit()



# -*- coding: UTF-8 -*-

import time
import traceback
from datetime import datetime

from tma.indicator import ShareDayIndicator
from tma.utils import is_in_trade_time
from tma import sms
from tma.utils import debug_print
from tma import logger


# 涨跌停板破板
# --------------------------------------------------------------------

def sm_limit(code, kind, threshold=10000, interval=1):
    """监控单只股票 涨停板买一挂单金额 / 跌停板卖一挂单金额

    :param code: str
        股票代码
    :param kind: str
        涨停板（zt） / 跌停板（dt）
    :param threshold: int, 默认值 10000
        金额阈值，单位：万元； 对于涨停板/跌停板来说，
        如果买一/卖一挂单金额小于阈值，说明有破板的可能，
        及时发送预警通知。
    :param interval: int, 默认值 1
        监控间隔，单位：秒
    :return: None
    """
    if kind == "zt":
        msg0 = "【涨停板 - 破板监控】"
        msg1 = "买一总挂单金额"
    elif kind == "dt":
        msg0 = "【跌停板 - 破板监控】"
        msg1 = "卖一总挂单金额"
    else:
        raise ValueError("kind 可选值为 ['zt', 'dt']，"
                         "当前值为 '%s'" % kind)

    start_info = "开始监控 - %s - %s | 参数配置：阈值（%i 万元）" \
                 % (msg0, str(code), threshold)
    debug_print(start_info)

    while is_in_trade_time():
        time.sleep(interval)
        try:
            sdi = ShareDayIndicator(code)
            sdi.cal_bs_first()

            if kind == "zt":
                money = sdi.features['BUY_FIRST']
            elif kind == "dt":
                money = sdi.features['SELL_FIRST']

            debug_info = "%s - %s - %s 万元" % (str(code), msg1, str(int(money / 10000)))
            debug_print(debug_info)

            if money / 10000 < threshold:
                title = "%s - %s 即将破板" % (msg0, str(code))
                content = "%s: %s万元，低于阈值（%s万元）" % (msg1,
                                                   str(int(money / 10000)), str(threshold))
                sms.server_chan_push(title, content)
        except Exception as e:
            traceback.print_exc()
            debug_print(str(e))
            continue

    end_info = "结束监控 - %s - %s | 参数配置：阈值（%i 万元）" \
               % (msg0, str(code), threshold)
    debug_print(end_info)


# 市场状态 & 个股行情
# --------------------------------------------------------------------

def get_shares_status(codes):
    raise NotImplementedError


def get_market_status():
    raise NotImplementedError


def fix_inform(codes, interval=1800):
    """交易时间段内，每隔一段时间播报一次市场状态和个股行情

    :param codes: list or str
        个股代码
    :param interval: int
        固定间隔时间，单位：秒
    :return: None
    """
    if not is_in_trade_time():
        return
    if isinstance(codes, str):
        codes = [codes]

    start_info = "启动 - 固定间隔播报 | 参数配置：个股列表（%s）、时间间隔（%s）" \
                 % (str(codes), str(interval))
    logger.info(start_info)

    shares = [ShareDayIndicator(code) for code in codes]
    share_status_template = "### {code}（{name}）\n --- \n" \
                            " 当前价格为{price}元，" \
                            "涨跌幅{change_rate}，振幅{wave_rate}，" \
                            "总成交金额{total_amount}万元。\n"

    while is_in_trade_time():
        status = []
        # 构造播报信息 - 市场

        # 构造播报信息 - 个股
        for share in shares:
            share.update(target=['bsf'])
            si = share.indicators
            share_status = share_status_template.format(
                code=si['CODE'], name=si['NAME'], price=si['PRICE'],
                change_rate=str(round(si['CHANGE_RATE'] * 100, 4)) + "%",
                wave_rate=str(round(si['WAVE_RATE'] * 100, 4)) + "%",
                total_amount=str(int(si['TOTAL_AMOUNT'] / 10000))
            )
            status.append(share_status)

        title = "市场状态播报 - %s" % datetime.now().__str__().split('.')[0]
        content = ''.join(status)
        sms.server_chan_push(title, content)
        time.sleep(interval)

    end_info = "结束 - 固定间隔播报 | 参数配置：个股列表（%s）、时间间隔（%s）" \
               % (str(codes), str(interval))
    logger.info(end_info)



# -*- coding: UTF-8 -*-

"""
collector.xueqiu - 采集雪球数据

官网：https://xueqiu.com/
====================================================================
"""

import requests
import time
from bs4 import BeautifulSoup
import webbrowser
from zb.crawlers.utils import get_header
import traceback

XUEQIU_HOME = "https://xueqiu.com/"


def make_symbol(code):
    if code.startswith("6"):
        symbol = "SH" + code
    elif code.startswith("3") or code.startswith("0"):
        symbol = "SZ" + code
    else:
        raise ValueError("构造雪球symbol失败，code应当以0/3/6开头")
    return symbol


def open_xueqiu(code):
    """打开code在雪球的页面

    :param code: 股票代码，如 603655
    :return:
    """
    symbol = make_symbol(code)
    url = "https://xueqiu.com/S/" + symbol
    webbrowser.open(url)


def get_comments(code, sleep=1):
    """获取股票code的雪球评论

    :param str code: 股票代码，如 `600122`
    :param float sleep: 睡眠时间， 默认值为 1
    :return:
    """
    headers = get_header()
    sess = requests.Session()

    # 访问首页，获取cookies
    sess.get(XUEQIU_HOME, headers=headers, timeout=10)

    # 获取首页评论
    symbol = make_symbol(code)
    real_time = str(time.time()).replace('.', '')[0:-1]  # 获取当前时间
    comment_url = 'https://xueqiu.com/statuses/search.json?' \
                  'count=10&comment=0&symbol={symbol}&hl=0&' \
                  'source=user&sort=time&page={page}&_={real_time}'
    url = comment_url.format(symbol=symbol, page=1, real_time=real_time)
    res = sess.get(url, headers=headers, timeout=10).json()
    count = res['count']            # 评论总条数
    total_page = res['maxPage']     # 页数
    print("总页数：", total_page)
    # 评论数据存储list
    comments = {
        "symbol": symbol,
        "count": count,
    }
    coms = []
    for i in range(1, total_page+1):
        print(i)
        time.sleep(sleep)
        headers = get_header()
        sess_temp = requests.Session()

        # 访问首页，获取cookies
        sess_temp.get(XUEQIU_HOME, headers=headers, timeout=10)
        real_time = str(time.time()).replace('.', '')[0:-1]  # 获取当前时间

        url = comment_url.format(symbol=symbol, page=i, real_time=real_time)
        try:
            res = sess_temp.get(url, headers=headers, timeout=10).json()['list']
        except Exception:
            print(i, "fail")
            traceback.print_exc()
            continue
        for r in res:
            com = {
                "text": BeautifulSoup(r['text'], 'lxml').text,
                "id": r['id'],
                "time": r['timeBefore'],
                "reply_count": int(r['reply_count']),
                "source": r['source']
            }

            user = {
                "id": r['user']['id'],
                "city": r['user']['city'],
                "description": r['user']['description'],
                "followers_count": r['user']['followers_count'],
                "friends_count": r['user']['friends_count'],
                "gender": r['user']['gender'],
                "nick_name": r['user']['screen_name'],
                "province": r['user']['province'],
                "status_count": r['user']['status_count'],
            }

            com['user'] = user

            if com['reply_count'] > 0:
                com['sub_comments'] = _get_sub_comments(com['id'])
            else:
                com['sub_comments'] = []

            coms.append(com)
    comments['comment_list'] = coms
    return comments


def _get_sub_comments(comment_id):
    """获取评论下面的子评论

    :param str comment_id: 评论id，如 `106580772`
    :return: list sub_comments
    """
    sub_com_url = "https://xueqiu.com/statuses/comments.json?id={comment_id}" \
                  "&count=20&page=1&reply=true&asc=false&type=status&split=true"
    url = sub_com_url.format(comment_id=comment_id)

    headers = get_header()
    sess = requests.Session()

    # 访问首页，获取cookies
    sess.get(XUEQIU_HOME, headers=headers, timeout=10)

    res = sess.get(url, headers=headers).json()['comments']
    sub_comments = []

    for r in res:
        com = {
            "timestamp": r['created_at'],
            "ip": r['created_ip'],
            "text": BeautifulSoup(r['text'], 'lxml').text,
            "source": r['source'],
        }

        user = {
            "id": r['user']['id'],
            "city": r['user']['city'],
            "description": r['user']['description'],
            "followers_count": r['user']['followers_count'],
            "friends_count": r['user']['friends_count'],
            "gender": r['user']['gender'],
            "nick_name": r['user']['screen_name'],
            "province": r['user']['province'],
            "status_count": r['user']['status_count'],
        }

        com["user"] = user

        sub_comments.append(com)
    return sub_comments


def get_top_portfolio(market='cn', profit="monthly_gain", count=30):
    """获取top实盘组合

    :param market:
    :param profit:
    :param count:
    :return:
    """
    base_url = "https://xueqiu.com/cubes/discover/rank/cube/list.json?" \
               "category=12&count={count}&market={market}&profit={profit}&sort=best_benefit"
    url = base_url.format(market=market, count=count, profit=profit)
    headers = get_header()
    sess = requests.session()
    sess.get(XUEQIU_HOME, headers=headers)
    res = sess.get(url, headers=headers).json()['list']

    top_pfs = []
    for r in res:
        pf = {
            "name": r['name'],
            "symbol": r['symbol'],
            "description": r['description'],
            "follower_count": r['follower_count'],
            "updated_at": r['updated_at'],
            "net_value": r['net_value'],
            "monthly_gain": str(r['monthly_gain'])+"%",
            "total_gain": str(r['annualized_gain_rate'])+"%",
            "last_rb_id": r['last_rb_id'],
        }

        user = {
            "id": r['owner']['id'],
            "city": r['owner']['city'],
            "description": r['owner']['description'],
            "followers_count": r['owner']['followers_count'],
            "friends_count": r['owner']['friends_count'],
            "gender": r['owner']['gender'],
            "nick_name": r['owner']['screen_name'],
            "province": r['owner']['province'],
            "status_count": r['owner']['status_count'],
        }

        pf['user'] = user

        top_pfs.append(pf)
    return top_pfs






# -*- coding: UTF-8 -*-

"""
indicator.share - 个股指标计算
====================================================================
"""
import numpy as np
from datetime import datetime
import traceback
from collections import OrderedDict

from tma.collector import klines, ticks, get_price, bars


class ShareDayIndicator(object):
    """以日为更新周期的个股指标体系"""

    def __init__(self, code):
        self.code = code
        self.features = OrderedDict({
            "DATE": datetime.now().date().__str__(),
            "CODE": code
        })
        self.kls = None
        self.tks = None
        self.bar = None
        self.update_time = {"kls": None,
                            'tks': None,
                            "bar": None,
                            "last_run": None}
        self.default_target = ('ma', 'lnd', 'bs', 'bsf')
        # self.basic_info()

    # 相关数据获取
    # --------------------------------------------------------------------

    def _get_kls(self, use_exists=True):
        """获取日K线"""
        if not use_exists:
            self.kls = klines(self.code, freq='D')
        if self.kls is None:
            kls = self.kls = klines(self.code, freq='D')
        else:
            kls = self.kls
        self.update_time['kls'] = datetime.now().__str__()
        return kls

    def _get_today_ticks(self, use_exists=True):
        """获取当日分笔"""
        if not use_exists:
            self.tks = ticks(self.code)
        if self.tks is None:
            tks = self.tks = ticks(self.code)
        else:
            tks = self.tks
        self.update_time['tks'] = datetime.now().__str__()
        return tks

    def _get_realtime_bar(self):
        """获取实时行情"""
        bar = self.bar = bars(self.code)
        self.update_time['bar'] = datetime.now().__str__()
        return bar

    # --------------------------------------------------------------------

    def basic_info(self):
        bar = self._get_realtime_bar()
        self.bar = bar.iloc[0]
        pre_close = float(bar.loc[0, 'pre_close'])
        # open_price = float(bar.loc[0, 'open'])
        high_price = float(bar.loc[0, 'high'])
        low_price = float(bar.loc[0, 'low'])
        cur_price = float(bar.loc[0, 'price'])

        BASIC = OrderedDict()
        BASIC['NAME'] = bar.loc[0, 'name']
        BASIC["PRICE"] = cur_price
        BASIC['TOTAL_AMOUNT'] = float(bar.loc[0, 'amount'])
        BASIC['CHANGE_RATE'] = (cur_price - pre_close) / pre_close
        BASIC['WAVE_RATE'] = (high_price - low_price) / pre_close
        self.features.update(BASIC)

    def cal_move_average(self, update=False):
        """计算移动均线指标"""
        if update or self.kls is None:
            kls = self.kls = self._get_kls()
        else:
            kls = self.kls
        seq_close = kls['close']
        MA = OrderedDict()
        MA.update({
            "MA5_D": np.mean(seq_close[-5:]),
            "MA10_D": np.mean(seq_close[-10:]),
            "MA20_D": np.mean(seq_close[-20:]),
            "MA30_D": np.mean(seq_close[-30:]),
            "MA60_D": np.mean(seq_close[-60:]),
            "MA120_D": np.mean(seq_close[-120:]),
            "MA240_D": np.mean(seq_close[-240:]),
        })
        for k, v in MA.items():
            MA[k] = round(v, 4)

        self.features.update(MA)

    def cal_latest_nd(self, d=None, update=False):
        """计算最近N个交易日的相关指标

        :param update:
        :param d: list, 默认 None
            指定N的大小，可以指定多个；如果为 None，则 d = [5, 10, 20, 40, 60]。
        :return:
        """
        if update or self.kls is None:
            kls = self.kls = self._get_kls()
        else:
            kls = self.kls
        if d is None:
            d = [5, 10, 20, 40, 60]
        kls['wave'] = kls['high'] - kls['low']
        kls['wave_rate'] = kls['wave'] / kls['open']
        LND = OrderedDict()
        for i in d:
            try:
                kls_d = kls[-i:]
                o = kls_d.iloc[0]['open']
                h = max(kls_d['high'])
                l = min(kls_d['low'])
                # 最高价
                LND["HIGH_" + str(i)] = h
                # 最低价
                LND["LOW_" + str(i)] = l
                # 涨跌幅
                LND["CHANGE_" + str(i)] = (kls_d.iloc[i - 1]['close'] - o) / o,
                # 最大回撤
                LND["MAX_DOWN_" + str(i)] = (kls_d.iloc[i - 1]['close'] - h) / h
                # 平均每日波动率
                LND["WAVE_RATE_A" + str(i)] = np.mean(kls_d['wave_rate'])
                # 累计波动率
                LND["WAVE_RATE_T" + str(i)] = (h - l) / o
            except Exception:
                traceback.print_exc()
        for k, v in LND.items():
            if isinstance(v, tuple):
                v = v[0]
            LND[k] = round(v, 4)

        self.features.update(LND)

    def cal_bs_dist(self, update=False):
        """计算今日分笔的买卖盘分布"""
        if update or self.tks is None:
            tks = self.tks = self._get_today_ticks()
        else:
            tks = self.tks
        tks['amount'] = tks['price'] * tks['vol'] * 100
        res = dict(tks.groupby('type').sum()['amount'])
        BS_DIST = OrderedDict()

        BS_DIST['BUY_AMOUNT'] = int(res.get(0, 0))
        BS_DIST['SELL_AMOUNT'] = int(res.get(1, 0))
        BS_DIST['NEUTRAL_AMOUNT'] = int(res.get(2, 0))

        self.features.update(BS_DIST)

    def cal_bs_first(self, update):
        """计算买一、卖一挂单金额"""
        if update or self.bar is None:
            bar = self._get_realtime_bar().iloc[0]
        else:
            bar = self.bar

        BS_FIRST = OrderedDict()

        # 卖一总挂单金额
        if not bar['b1_v']:
            BS_FIRST['BUY_FIRST'] = 0
        else:
            BS_FIRST['BUY_FIRST'] = float(bar['b1_v']) * float(bar['b1_p']) * 100

        # 买一总挂单金额
        if not bar['a1_v']:
            BS_FIRST['SELL_FIRST'] = 0
        else:
            BS_FIRST['SELL_FIRST'] = float(bar['a1_v']) * float(bar['a1_p']) * 100

        self.features.update(BS_FIRST)

    def update(self, target=None):
        """更新个股指标"""
        if not target:
            target = self.default_target
        self.run(target=target, update=True)

    def run(self, target=None, update=False):
        """计算所有个股相关指标的主函数，
        继承ShareIndicator对象，重写run方法可以自由选择计算哪些指标
        """
        self.basic_info()
        if not target:
            target = self.default_target
        funcs = {
            "ma": self.cal_move_average,
            "lnd": self.cal_latest_nd,
            "bs": self.cal_bs_dist,
            "bsf": self.cal_bs_first
        }
        for x in target:
            if x in funcs.keys():
                funcs[x](update=update)
            else:
                raise ValueError('%s 不是合法的指标关键词' % x)
        # 记录最后一次运行的时间
        self.update_time['last_run'] = datetime.now().__str__()

    @property
    def indicators(self):
        indicators = OrderedDict()
        for k, v in self.features.items():
            if isinstance(v, float):
                indicators[k] = round(v, 4)
            else:
                indicators[k] = v
        return indicators


class ShareWeekIndicator(object):
    """以周为更新周期的个股指标体系"""

    def __init__(self):
        pass



# -*- coding: UTF-8 -*-

"""
indicator.market - A股市场指标计算
====================================================================
"""

import pandas as pd
from collections import OrderedDict

from tma.collector import today_market
from tma.indicator.meta import check_indicator_meta


class MarketIndicator(object):
    """A股全市场单个交易日的指标体系"""
    def __init__(self):
        self.features = OrderedDict()
        self.m = None

    # 数据获取
    # --------------------------------------------------
    def _get_market(self, use_exists=True):
        if not use_exists:
            self.m = today_market(filters=['tp'], use_latest=False)
        if self.m is None:
            m = self.m = today_market(filters=['tp'], use_latest=True)
        else:
            m = self.m
        return m

    # 指标计算
    # --------------------------------------------------
    @staticmethod
    def _up_rate(m):
        """计算赚钱效应相关指标"""
        total = len(m)
        m['is_up'] = m['changepercent'].apply(lambda x: True if x > 0.0 else False)
        m['is_up_3'] = m['changepercent'].apply(lambda x: True if x > 3.0 else False)
        m['is_down_3'] = m['changepercent'].apply(lambda x: True if x < -3.0 else False)
        up3 = dict(m['is_up_3'].value_counts())[True]
        up = dict(m['is_up'].value_counts())[True]
        down3 = dict(m['is_down_3'].value_counts())[True]
        return total, up, up3, down3

    def cal_total_market(self):
        """计算全市场赚钱效应相关指标"""
        m = self._get_market()
        total, up, up3, down3 = self._up_rate(m)
        f = OrderedDict(
            {
                "M001": up / total,
                "M002": total,
                "M003": up,
                "M004": up3,
                "M005": down3,
            }
        )
        self.features.update(f)

    def cal_turnover_top50(self):
        """计算换手率前50只股票的赚钱效应相关指标"""
        m = self._get_market()
        m.sort_values('turnoverratio', inplace=True)
        m.reset_index(drop=True, inplace=True)
        m_tt50 = m.tail(50)
        total, up, up3, down3 = self._up_rate(m_tt50)
        f = OrderedDict(
            {
                "M006": up / total,
                "M007": up3,
                "M008": down3,
            }
        )
        self.features.update(f)

    def cal_limit_arrived(self):
        """计算涨跌停板相关指标"""
        m = self._get_market()
        m = m[[
            'code', 'name', 'changepercent', 'trade',
            'open', 'high', 'low', 'settlement', 'volume'
        ]]
        la = []
        for i in m.index:
            data = m.loc[i]
            if data['volume'] == 0.0:
                continue

            # 涨停板观察
            if data['high'] > data['settlement'] * 1.095:
                if data['high'] > data['trade']:  # 盘中触及涨停板
                    x = dict(data)
                    x['kind'] = "盘中触及涨停板"
                    la.append(x)
                elif data['high'] == data['low']:  # 一字涨停板
                    x = dict(data)
                    x['kind'] = "一字涨停板"
                    la.append(x)
                elif data['high'] == data['trade']:  # 涨停板
                    x = dict(data)
                    x['kind'] = "涨停板"
                    la.append(x)
                else:
                    continue

            # 跌停板观察
            if data['low'] < data['settlement'] * 0.905:
                if data['trade'] > data['low']:  # 盘中触及跌停板
                    x = dict(data)
                    x['kind'] = "盘中触及跌停板"
                    la.append(x)
                elif data['high'] == data['low']:  # 一字跌停板
                    x = dict(data)
                    x['kind'] = "一字跌停板"
                    la.append(x)
                elif data['low'] == data['trade']:  # 跌停板
                    x = dict(data)
                    x['kind'] = "跌停板"
                    la.append(x)
                else:
                    continue

        df_la = pd.DataFrame(la)
        df_la = df_la[['code', 'name', 'trade', 'open', 'high', 'low',
                        'kind', 'changepercent']]
        df_la = df_la.sort_values('kind')

        la_count = OrderedDict(df_la['kind'].value_counts())
        x1 = la_count.get("涨停板", 0) + la_count.get("一字涨停板", 0)
        x2 = la_count.get("一字涨停板", 0)
        x3 = la_count.get("盘中触及涨停板", 0)
        x4 = la_count.get("跌停板", 0) + la_count.get("一字跌停板", 0)
        x5 = la_count.get("一字跌停板", 0)
        x6 = la_count.get("盘中触及跌停板", 0)

        f = OrderedDict(
            {
                "M009": x1,
                "M010": x2,
                "M011": x3,
                "M013": x4,
                "M014": x5,
                "M015": x6
            }
        )
        f['M012'] = x1 / (x1 + x3) if x1 + x3 != 0 else 0
        f['M016'] = x4 / (x4 + x6) if x4 + x6 != 0 else 0

        self.features.update(f)

    # 其他
    # --------------------------------------------------
    @property
    def indicators(self):
        indicators = OrderedDict()
        for k, v in self.features.items():
            indicators[k] = {
                "value": round(v, 4),
                "explain": check_indicator_meta(k)['explain']
            }
        return indicators

    def update(self):
        self._get_market(use_exists=False)
        self.run()

    def run(self):
        self._get_market()
        self.cal_total_market()
        self.cal_turnover_top50()
        self.cal_limit_arrived()






# -*- coding: UTF-8 -*-

"""
analyst.rank - 按照不同的标准对股票进行排序
====================================================================
"""

from tma.collector.aggregation import agg_market_klines


class BaseRank:
    """基类"""

    def __init__(self, name, desc):
        self.name = name
        self.desc = desc

    def rank(self, top=None):
        raise NotImplementedError

    @property
    def top30(self):
        return self.rank(top=30)

    @property
    def top50(self):
        return self.rank(top=50)

    @property
    def top100(self):
        return self.rank(top=100)


class WeekTop(BaseRank):
    """

    排序准则：周涨幅 * 0.5 + 周振幅 * 0.5

    TODO: 优化 - 剔除上市不到30天的新股
    """

    def __init__(self, date, refresh=False):
        desc = "以涨幅和振幅为依据的周排序方法"
        super().__init__(name='week_top', desc=desc)
        # 参数
        self.date = date
        self.refresh = refresh
        # 数据及计算结果
        self.latest_mkls = None

    def data_prepare(self):
        """获取排序需要的数据，按照排序准则进行计算"""
        mkls = agg_market_klines(k_freq='W',
                                 refresh=self.refresh)
        latest_mkls = mkls[mkls["date"] == self.date]
        latest_mkls["change"] = latest_mkls["close"] - latest_mkls['open']
        latest_mkls["change_rate"] = latest_mkls["change"] / latest_mkls["open"]
        latest_mkls["wave"] = latest_mkls["high"] - latest_mkls['low']
        latest_mkls["wave_rate"] = latest_mkls["wave"] / latest_mkls["low"]
        latest_mkls['criterion'] = 0.5 * latest_mkls["change_rate"] + \
                                   0.5 * latest_mkls["wave_rate"]
        latest_mkls.drop(['open', 'close', 'high', 'low',
                          'volume', 'change', 'wave', ], axis=1, inplace=True)
        latest_mkls.sort_values('criterion', ascending=False, inplace=True)
        self.latest_mkls = latest_mkls.reset_index(drop=True)

    def rank(self, top=None):
        self.data_prepare()
        if top is None:
            top_shares = list(self.latest_mkls['code'])
        else:
            indexes = range(top)
            top_shares = list(self.latest_mkls.loc[indexes, 'code'])
        return list(enumerate(top_shares, 1))





# -*- coding: UTF-8 -*-

"""
Tushare数据接口封装
====================================================================
"""

import os
import time
from datetime import datetime

import pandas as pd
import tushare as ts

from tma import DATA_PATH


def get_market_basic():
    """返回A股所有股票的基础信息"""
    return ts.get_stock_basics()


def index_all():
    """指数行情接口"""
    return ts.get_index()


def get_price(code):
    """获取一只股票的最新价格

    :param code: str
        股票代码，如：600122
    :return: float
    """
    data = ts.get_realtime_quotes(code)
    return float(data.loc[0, 'price'])


def get_ticks(code, source="spider", date=None, cons=None):
    """返回date日期的分笔数据

    :param source:
    :param code: str: 股票代码，如 603655
    :param date: str: 日期，如 2018-03-15
    :param cons: tushare的api连接
    :return:
    """
    if not date:
        date = datetime.now().date().__str__()
    TODAY = datetime.now().date().__str__()

    # 统一 ticks 的输出结果
    def _unify_out(ticks, date):
        ticks = ticks[['time', 'price', 'volume', 'type']]
        ticks['datetime'] = ticks['time'].apply(lambda x: datetime.strptime(date + " " + x, "%Y-%m-%d %H:%M:%S"))
        ticks['vol'] = ticks['volume']
        type_convert = {
            "买盘": 0,
            "卖盘": 1,
            "中性盘": 2,
            "0": 2
        }
        ticks['type'] = ticks["type"].apply(lambda x: type_convert[str(x)])
        ticks.drop(['time', 'volume'], axis=1, inplace=True)
        ticks.sort_values('datetime', inplace=True)
        ticks.reset_index(drop=True, inplace=True)
        return ticks[['datetime', 'price', 'vol', 'type']]

    if source == "spider" and date == TODAY:
        ticks = ts.get_today_ticks(code=code)
        ticks = _unify_out(ticks, date=TODAY)
    elif source == "spider" and date != TODAY:
        ticks = ts.get_tick_data(code=code, date=date)
        ticks = _unify_out(ticks, date=date)
    else:
        if not cons:
            cons = ts.get_apis()
        ticks = ts.tick(code=code, conn=cons, date=date)
    return ticks


ticks = get_ticks


def get_bars(codes):
    """获取codes的实时quotes"""
    return ts.get_realtime_quotes(codes)


bars = get_bars


def get_klines(code, freq="D", start_date=None):
    """获取K线数据

    :param code: str 股票代码
    :param start_date: str 默认为 None
        开始日期。值为None时，默认获取全部。
    :param freq: str 默认为 "D"
        K线周期，可选值 D=日k线 W=周 M=月 5=5分钟 15=15分钟 30=30分钟 60=60分钟
    :return: pd.DataFrame
    """
    if start_date is None:
        data = ts.get_k_data(code=code, ktype=freq)
    else:
        data = ts.get_k_data(code=code, start=start_date, ktype=freq)
    return data


klines = get_klines


def filter_tp(tm):
    """停盘股过滤

    :param tm: return of function today_market
    :return:
    """
    tm1 = tm[tm['volume'] != 0.0]
    tm1.reset_index(drop=True, inplace=True)
    return tm1


def filter_st(tm):
    """ST股过滤

    :param tm: return of function today_market
    :return:
    """
    fst = tm['name'].apply(lambda x: True if "ST" not in x else False)
    tm1 = tm[fst]
    tm1.reset_index(drop=True, inplace=True)
    return tm1


def get_today_market(filters=None, save=True,
                     use_latest=False, interval=600):
    """返回最近一个交易日所有股票的交易数据

    :param filters: list 默认为 ['tp']
        需要应用的过滤规则，可选 "tp"、"st"。tp表示过滤停盘股，st表示过滤st股。
    :param save: bool 默认为 True
        是否缓存到user目录下
    :param use_latest: bool 默认为 False
        是否使用最近缓存的数据
    :param interval: int 默认 600
        更新行情的最小间隔（单位：s），即：如果DATA_PATH路径下的latest_market的修改时间
        与当前时间的间隔小于interval设定的数值，且use_latest为True，
        将使用latest_market.csv中的行情
    :return: pd.DataFrame
        最新的市场行情
    """
    if filters is None:
        filters = ['tp']
    tm_csv = os.path.join(DATA_PATH, 'latest_market.csv')
    if use_latest and os.path.exists(tm_csv) \
            and time.time() - os.path.getmtime(tm_csv) < interval:
        tm = pd.read_csv(tm_csv, encoding='utf-8')
        return tm

    tm = ts.get_today_all()
    if filters is None:
        return tm
    filters = [x.lower() for x in filters]
    if "tp" in filters:
        tm = filter_tp(tm)
    if "st" in filters:
        tm = filter_st(tm)
    if save:
        tm.to_csv(tm_csv, index=False, encoding='utf-8')
    return tm


today_market = get_today_market


def get_hist_market(date):
    """历史行情数据

    :param date: str:
        指定日期，如 "2018-03-19"
    :return:
    """
    hm = ts.get_day_all(date)
    hm['date'] = date
    return hm


hist_market = get_hist_market



"""
StockPool - 股票池对象
====================================================================
"""
import os
import traceback
from zb.tools import file_tools as ft

from tma import POOL_PATH


def _verify_share(share):
    base = {
        "code": None,
        "name": None,
        "level": None,  # 1, 2, 3
        "date": None,  # 入选日期
        "reason": None,  # 入选理由
        "price": None  # 入选时的价格
    }
    try:
        assert share.keys() == base.keys(), \
            "share中应当包含这些key：%s" % str(base.keys())
        assert share['level'] in [1, 2, 3], \
            "level取值必须是[1, 2, 3]中的一个，当前值：%s" % str(share['level'])
        return True
    except:
        return False


class StockPool:
    def __init__(self, name):
        self.name = name

        # 股票池保存路径, 以.pool为后缀
        self.path = os.path.join(POOL_PATH, '%s.pool' % self.name)
        self.path_hist = self.path.replace(".pool", "_hist.pool")

        self.level1 = {}
        self.level2 = {}
        self.level3 = {}
        self._read_pool()

    def _read_pool(self):
        """读入股票池文件"""
        path = self.path
        if not os.path.exists(path):
            ft.create_file(path)
            ft.create_file(self.path_hist)
        else:
            self.level1 = {}
            self.level2 = {}
            self.level3 = {}
            shares = [eval(share.strip('\n')) for share in ft.read_file(path)]
            for share in shares:
                if share['level'] == '1':
                    if share['code'] not in self.level1.keys():
                        self.level1[share['code']] = []
                    self.level1[share['code']].append(share)

                elif share['level'] == '2':
                    if share['code'] not in self.level2.keys():
                        self.level2[share['code']] = []
                    self.level2[share['code']].append(share)

                elif share['level'] == '3':
                    if share['code'] not in self.level3.keys():
                        self.level3[share['code']] = []
                    self.level3[share['code']].append(share)

                else:
                    print("error: ", share)

    def add(self, share, read=True):
        """添加单只股票

        每只股票可以多个入选理由，因此存在多条记录。

        """
        try:
            _verify_share(share)
            ft.write_file(self.path, str(share), mode='a')
            if read:
                self._read_pool()
        except Exception:
            traceback.print_exc()

    def add_many(self, shares):
        """添加多只股票"""
        for share in shares:
            self.add(share, read=False)
        self._read_pool()

    def remove(self, code, level, save_to_hist=True):
        """指定股票代码，删除股票

        不管该股票有多少条记录，都会一次性全部删除

        """
        try:
            shares = [eval(share.strip('\n')) for share in ft.read_file(self.path)]
            shares_keep = []
            shares_hist = []
            for share in shares:
                if share['code'] == code and share['level'] == level:
                    shares_hist.append(share)
                else:
                    shares_keep.append(share)
            ft.write_file(self.path, shares_keep, mode='w')
            if save_to_hist:
                ft.write_file(self.path_hist, shares_keep, mode='a')
            self._read_pool()
        except Exception:
            traceback.print_exc()

    def delete(self, code, level):
        self.remove(code=code, level=level, save_to_hist=False)

    def empty(self):
        """清空股票池"""
        shares = ft.read_file(self.path)
        ft.write_file(self.path_hist, shares, mode='a')
        ft.create_file(self.path, mode='w')
        self._read_pool()



# -*- coding: UTF-8 -*-

"""
collector - 数据采集模块
1）tushare数据接口封装；
2）雪球数据采集
3）同花顺数据采集
====================================================================
"""


# tushare数据接口封装
from .ts import (klines, bars, ticks, today_market, get_price)

# 上海证券交易所官网采集数据接口
from .sse import get_sh_indexes





# -*- coding: utf-8 -*-
"""
rules - A股投资纪律
====================================================================
"""

from collections import OrderedDict

RULES = OrderedDict()

RULES["META"] = {
    "author": "zengbin",
    "create_date": "2018-07-04",
    "introduction": "1、纪律性是投资不可或缺的一部分！在这里，我记录了"
                    "一些广受认可的A股投资纪律，仅供参考。\n"
                    "2、纪律之间难免会有一些矛盾冲突的部分，为此，我根"
                    "据自身有限的投资经验，将这些记录分了不同的等级"
                    "（1~3，重要性逐级递减），如果两条纪律之间发生"
                    "冲突，优先准守高等级记录。\n"
                    "3、仅适用于个人投资者。\n",
    "contact": "zeng_bin8888@163.com"
}

# --------------------------------------------------------------------
RULES['R001'] = {
    "rule": "把更多的时间花在大趋势感知和选股上面，而不是研究交易策略，"
            "更不是盯盘！",
    "explain": "1、在趋势向上的时候选中一只好股，不管用什么策略去交易，"
               "都可以赚点钱；反过来，再牛逼的交易策略，在行情不好的"
               "时候用在一只烂股上，大概率亏钱。当然，交易策略的研究"
               "也是非常有必要的。\n"
               "2、永远跟着大趋势走，不要逆趋势而动；别对垃圾股抱有"
               "侥幸心理，切忌自我麻痹。\n"
               "3、行情（趋势）不好的时候，什么都有可能发生。\n",
    "level": 1,
    "create_date": "2018-07-04",
    "update_date": []
}

# --------------------------------------------------------------------
RULES['R002'] = {
    "rule": "不要试图抄底，更不能盲目追高！",
    "explain": "1、精准抄底这种事情是不存在的，没有谁可以预先知道哪里是底部；"
               "抄底这个行为本身就一个毒瘤，往往是造成亏损的主因，低点后面"
               "通常还有低点。\n"
               "2、追高的一个显著缺陷是潜在收益远大于风险。\n"
               "3、应用这条规则，你必须首先对“底”和“高”有一个较为清晰的定义。\n",
    "level": 1,
    "create_date": "2018-07-04",
    "update_date": []
}

# --------------------------------------------------------------------
RULES["R003"] = {
    "rule": "交易时间段，应该多关注股票池中的交易机会；不要进行选股操作，更不"
            "能买入非股票池中股票！",
    "explain": "1、非交易时间段，更容易保持理性的思维，在这个时候进行复盘、选股"
               "往往不容易犯错。\n"
               "2、股票池可能认为是一种非常有效的保险机制，贸然买入非股票池的"
               "股票，不确定性风险很大，花大量的时间维护一个高度可靠的股票"
               "池是非常值得的！\n",
    "level": 1,
    "create_date": "2018-07-04",
    "update_date": []
}

# --------------------------------------------------------------------
RULES["R004"] = {
    "rule": "宁可错过，不能做错！",
    "explain": "1、股市机会千千万，错过一个，还有下一个。\n"
               "2、如果你因错过一个机会而遗憾，导致情绪失控（失去理性），做出"
               "错误的交易行为，大概率会造成损失。多花点时间研究市场，不要去"
               "做错误的事情！\n",
    "level": 1,
    "create_date": "2018-07-04",
    "update_date": []
}


# --------------------------------------------------------------------

class Rules:
    def __init__(self):
        self.RULES = RULES
        self.rule_ids = list(self.RULES.keys()).remove('META')

    def print_rules(self):
        rules = [k + ": " + v['rule'] + "(" + str(v['level']) + ")\n"
                 for k, v in self.RULES.items() if k != "META"]
        for rule in rules:
            print(rule)

    def explain_rule(self, rule_id):
        rule = self.RULES[rule_id]
        print(rule_id + ": " + rule['rule'] + "(" + str(rule['level']) + ")\n")
        print("解释如下：")
        print(rule['explain'])

    @property
    def amount(self):
        return len(self.rule_ids)



# -*- coding: UTF-8 -*-

"""
analyst.classify - 对交易数据进行分类分析
====================================================================
"""


class ShareTrendClassifier(object):
    """个股走势分类器"""
    pass


class IndexTrendClassifier(object):
    """指数走势分类器"""
    pass



# coding: utf-8

import os
import unittest

from tma import pool

class StockPoolTest(unittest.TestCase):
    def setUp(self):
        path = "test.pool"
        if os.path.exists(path):
            os.remove(path)
        self.pool = pool.StockPool(path)

    def tearDown(self):
        os.remove(self.pool.path)
        os.remove(self.pool.path_hist)

    def test_add(self):
        share = {
            "code": '600122',
            "name": '宏图高科',
            "level": '3',  # 1, 2, 3
            "date": "2018-05-27",  # 入选日期
            "reason": "sb",  # 入选理由
            "price": "20"  # 入选时的价格
        }
        self.pool.add(share)
        self.assertDictEqual(share, self.pool.level3['600122'][0])

    def test_add_many(self):
        shares = [
            {
                "code": '600122',
                "name": '宏图高科',
                "level": '1',  # 1, 2, 3
                "date": "2018-05-27",  # 入选日期
                "reason": "sb",  # 入选理由
                "price": "20"  # 入选时的价格
            },
            {
                "code": '600122',
                "name": '宏图高科',
                "level": '2',  # 1, 2, 3
                "date": "2018-05-27",  # 入选日期
                "reason": "sb",  # 入选理由
                "price": "20"  # 入选时的价格
            },
            {
                "code": '600122',
                "name": '宏图高科',
                "level": '3',  # 1, 2, 3
                "date": "2018-05-27",  # 入选日期
                "reason": "sb",  # 入选理由
                "price": "20"  # 入选时的价格
            },
            {
                "code": '600122',
                "name": '宏图高科',
                "level": '3',  # 1, 2, 3
                "date": "2018-05-27",  # 入选日期
                "reason": "sb",  # 入选理由
                "price": "20"  # 入选时的价格
            },
            {
                "code": '600122',
                "name": '宏图高科',
                "level": '3',  # 1, 2, 3
                "date": "2018-05-27",  # 入选日期
                "reason": "sb",  # 入选理由
                "price": "20"  # 入选时的价格
            }
        ]
        self.pool.add_many(shares)
        self.assertEqual(3, len(self.pool.level3['600122']))
        self.assertDictEqual(shares[0], self.pool.level1['600122'][0])
        self.assertDictEqual(shares[1], self.pool.level2['600122'][0])

    def test_remove(self):
        share = {
            "code": '600122',
            "name": '宏图高科',
            "level": '3',  # 1, 2, 3
            "date": "2018-05-27",  # 入选日期
            "reason": "sb",  # 入选理由
            "price": "20"  # 入选时的价格
        }
        self.pool.add(share)
        self.assertDictEqual(share, self.pool.level3['600122'][0])
        self.pool.remove(code='600122', level='3')
        self.assertEqual(self.pool.level3, {})

    def test_empty(self):
        shares = [
            {
                "code": '600122',
                "name": '宏图高科',
                "level": '1',  # 1, 2, 3
                "date": "2018-05-27",  # 入选日期
                "reason": "sb",  # 入选理由
                "price": "20"  # 入选时的价格
            },
            {
                "code": '600122',
                "name": '宏图高科',
                "level": '2',  # 1, 2, 3
                "date": "2018-05-27",  # 入选日期
                "reason": "sb",  # 入选理由
                "price": "20"  # 入选时的价格
            },
            {
                "code": '600122',
                "name": '宏图高科',
                "level": '3',  # 1, 2, 3
                "date": "2018-05-27",  # 入选日期
                "reason": "sb",  # 入选理由
                "price": "20"  # 入选时的价格
            },
            {
                "code": '600122',
                "name": '宏图高科',
                "level": '3',  # 1, 2, 3
                "date": "2018-05-27",  # 入选日期
                "reason": "sb",  # 入选理由
                "price": "20"  # 入选时的价格
            },
            {
                "code": '600122',
                "name": '宏图高科',
                "level": '3',  # 1, 2, 3
                "date": "2018-05-27",  # 入选日期
                "reason": "sb",  # 入选理由
                "price": "20"  # 入选时的价格
            }
        ]
        self.pool.add_many(shares)
        self.pool.empty()
        self.assertEqual(self.pool.level3, {})
        self.assertEqual(self.pool.level2, {})
        self.assertEqual(self.pool.level1, {})
        # self.assertEqual(self.pool.is_empty(), True)

if __name__ == "__main__":
    unittest.main()



# coding: utf-8

"""
collector.xhn - 新华网数据采集

官网：http://www.xinhuanet.com/

接口分析：
1. 获取文章列表
http://qc.wa.news.cn/nodeart/list?nid=115093&pgnum=1&cnt=10000

新华全媒体头条
http://www.xinhuanet.com/politics/qmtt/index.htm
====================================================================
"""

import requests
import re
from datetime import datetime
from bs4 import BeautifulSoup
from zb.crawlers.utils import get_header
import traceback
import pandas as pd

import tma

home_url = "http://www.xinhuanet.com/"


def get_website_map():
    wzdt_url = "http://www.xinhuanet.com/wzdt2014.htm"
    html = requests.get(wzdt_url, headers=get_header())
    bsobj = BeautifulSoup(html.content.decode('utf-8'), 'lxml')
    map_raw = bsobj.find('div', {'class': "content_left"})
    raise NotImplementedError


def get_special_topics(pgnum=1):
    """获取专题列表"""
    url = "http://qc.wa.news.cn/nodeart/list?" \
          "nid=115093&pgnum=%s&cnt=200" % str(pgnum)
    res = requests.get(url).text
    res = res.replace("null", "\'\'")
    res = eval(res)
    assert res['status'] == 0, "获取文章列表失败"
    data = res['data']['list']
    specials = []
    for a in data:
        special = {
            "Abstract": a['Abstract'],
            "Author": a['Author'],
            "LinkUrl": a['LinkUrl'],
            "PubTime": a['PubTime'],
            "Title": a['Title'],
            "allPics": a['allPics'],
        }
        specials.append(special)
    return specials


def get_article_detail(article_url):
    """获取新华网article_url中的文章内容

    :param article_url: 文章url
    :return:
        {
        "url": article_url,
        "title": title,
        "pub_time": pub_time,
        "source": source,
        "content": content
        }
    """
    # article_url = "http://www.xinhuanet.com/fortune/2018-06/20/c_129897476.htm"
    html = requests.get(article_url, headers=get_header())
    bsobj = BeautifulSoup(html.content.decode('utf-8'), 'lxml')

    # 解析标题
    cols = bsobj.find('div', {"class": "h-news"}).text.strip().split("\r\n")
    title = cols[0].strip()
    pub_time = cols[1].strip()
    source = cols[-1].strip()

    # 解析内容
    content = bsobj.find('div', {"id": "p-detail"}).text.strip()
    content = content.replace("\u3000\u3000", "")
    content = [x.strip() for x in content.split("\n")]
    content = [x for x in content if x != ""]
    content = "\n".join(content)

    return {
        "url": article_url,
        "title": title,
        "pub_time": pub_time,
        "source": source,
        "content": content
    }


class HomePage(object):
    """新华网首页"""

    def __init__(self):
        self.home_url = "http://www.xinhuanet.com/"

    @staticmethod
    def _get_date_from_url(url):
        pat = re.compile("(\d{4}-\d{2}[/-]\d{2})")
        res = pat.findall(url)
        if res is not None and len(res) == 1:
            return res[0].replace('/', "-")
        else:
            return None

    def get_article_list(self):
        """获取首页的头条文章列表"""
        html = requests.get(self.home_url, headers=get_header())
        bsobj = BeautifulSoup(html.content.decode('utf-8'), 'lxml')

        a_list = []
        for a in bsobj.find_all("a"):
            try:
                url = a['href']
                title = a.text.strip()
                d = self._get_date_from_url(url)
                a_list.append([url, title, d])
            except:
                if tma.DEBUG:
                    traceback.print_exc()
                continue

        a_list = [a for a in a_list if
                  a[0] != ""
                  and a[0].strip("/") != "http://xhgy.xinhuanet.com"
                  and a[0].startswith("http")
                  and a[1] != ""
                  and a[1] != "视频MP4地址"
                  and "c_" in a[0]
                  and 'photo' not in a[0]
                  and 'video' not in a[0]
                  ]

        # 根据url去重
        df = pd.DataFrame(a_list, columns=['url', 'title', 'date'])
        df.drop_duplicates('url', inplace=True)
        res = [list(x) for x in list(df.values)]
        return res

    def get_articles(self, d=None):
        """获取首页文章内容

        :param d: list
            限定获取文章的日期，默认是当日日期，可以指定多个离散的日期
        :return: list
        """
        if d is None:
            d = [datetime.now().date().__str__()]

        # 获取首页文章列表URL、按发布日期过滤、按URL去重
        a_list = self.get_article_list()
        a_list = [a[0] for a in a_list if a[2] in d]
        a_list = list(set(a_list))

        articles = []
        for a in a_list:
            try:
                article = get_article_detail(a)
                articles.append(article)
            except:
                if tma.DEBUG:
                    traceback.print_exc()
                continue
        return articles


class Fortune(object):
    def __init__(self):
        self.url1 = "http://www.xinhuanet.com/fortune/"
        self.url2 = "http://www.xinhuanet.com/fortune/caiyan.htm"
        self.url3 = "http://www.xinhuanet.com/fortune/cfx.htm"
        self.url4 = "http://www.xinhuanet.com/fortune/bcxc.htm"



# -*- coding: UTF-8 -*-

"""
indicator.meta - 指标信息（元标签）
====================================================================
"""

from collections import OrderedDict

# A股 Share 指标体系 - 日
# 编码规则：S + 三位数字（001~999）
# --------------------------------------------------------------------
SHARE_INDICATOR_META = OrderedDict({
    "create_date": '2018-06-30',
    "creator": "zengbin",
    "explain": "单只股票指标体系 - 用于刻画个股状态",
    "update": [
        # 更新记录：日期 - 更改人 - 变动内容
    ],
})

# A股 Market 指标体系 - 日
# 编码规则：M + 三位数字（001~999）
# --------------------------------------------------------------------

MARKET_INDICATOR_META = OrderedDict()

MARKET_INDICATOR_META['INFO'] = OrderedDict({
    "create_date": '2018-06-20',
    "creator": "zengbin",
    "explain": "A股全市场单个交易日的指标体系 - 用于刻画市场状态，感知趋势变化",
    "update": [
        # 更新记录：日期 - 更改人 - 变动内容
    ],
})

MARKET_INDICATOR_META['M001'] = OrderedDict({
    'explain': "全市场赚钱效应",
    "cal_method": "全市场上涨个股数量 / 开盘个股总数"
})

MARKET_INDICATOR_META['M002'] = OrderedDict({
    'explain': "开盘个股数量",
})

MARKET_INDICATOR_META['M003'] = OrderedDict({
    'explain': "上涨个股数量"
})

MARKET_INDICATOR_META['M004'] = OrderedDict({
    'explain': "涨3个点以上个股数量"
})

MARKET_INDICATOR_META['M005'] = OrderedDict({
    "explain": "跌3个点以上个股数量"
})

MARKET_INDICATOR_META['M006'] = OrderedDict({
    "explain": "换手率前五十只股票的赚钱效应",
    "cal_method": "换手率前五十只股票中上涨个股数量 / 50",
})

MARKET_INDICATOR_META['M007'] = OrderedDict({
    "explain": "换手率前五十只股票中涨3个点以上的数量"
})

MARKET_INDICATOR_META['M008'] = OrderedDict({
    "explain": "换手率前五十只股票中跌3个点以上的数量"
})

MARKET_INDICATOR_META['M009'] = OrderedDict({
    "explain": "涨停板个股数量",
    "cal_method": "（涨幅 > 0.095）的个股总数",
})

MARKET_INDICATOR_META['M010'] = OrderedDict({
    "explain": "一字涨停板个股数量",
    "cal_method": "（涨幅 > 0.095）且（最高价 == 最低价）的个股总数",
})

MARKET_INDICATOR_META['M011'] = OrderedDict({
    "explain": "盘中触及涨停板个股数量",
    "cal_method": "（最高价涨幅 > 0.095）且（现价 < 最高价）的个股总数",
})

MARKET_INDICATOR_META['M012'] = OrderedDict({
    "explain": "涨停封板成功率",
    "cal_method": "涨停板个股数量 / （涨停板个股数量 + 盘中触及涨停板个股数量）",
})

MARKET_INDICATOR_META['M013'] = OrderedDict({
    "explain": "跌停板个股数量",
    "cal_method": "（跌幅 > 0.095）的个股总数",
})

MARKET_INDICATOR_META['M014'] = OrderedDict({
    "explain": "一字跌停板个股数量",
    "cal_method": "（跌幅 > 0.095）且（最高价 == 最低价）的个股总数",
})

MARKET_INDICATOR_META['M015'] = OrderedDict({
    "explain": "盘中触及跌停板个股数量",
    "cal_method": "（最低价跌幅 > 0.095）且（现价 > 最低价）的个股总数",
})

MARKET_INDICATOR_META['M016'] = OrderedDict({
    "explain": "跌停封板成功率",
    "cal_method": "跌停板个股数量 / （跌停板个股数量 + 盘中触及跌停板个股数量）",
})


# 查询 Indicator 元信息
# --------------------------------------------------------------------

def check_indicator_meta(indicator=None):
    """查询 indicator 的元信息"""
    if indicator.startswith("M"):
        return MARKET_INDICATOR_META.get(indicator, None)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

import tma


setup(
    name=tma.__name__,
    version=tma.__version__,
    keywords=(
        "A股", "xueqiu", "雪球", "TuShare", "交易数据分析", "三级股票池", "仿真交易",
        "A股数据采集"
    ),
    description="Tools for Market A - A股工具集",
    long_description="专为中国A股设计开发的工具集，着重点是交易数据获取和分析。我是一只菜鸟，对A股"
                     "的认识非常肤浅，如果你愿意用积攒的经验为我开发这套工具提供一些帮助，"
                     "请联系 zeng_bin8888@163.com ，不胜感激！",
    license="MIT License",
    url="https://github.com/zengbin93/TMA",
    author=tma.__author__,
    author_email="zeng_bin8888@163.com",
    packages=find_packages(exclude=['test', 'doc', 'img']),
    include_package_data=True,
    install_requires=[
        "tushare", "pandas", "requests", "zb", "retrying", "numpy",
        "bs4"
    ],
    python_requires=">=3",
    entry_points={}
)





# -*- coding: UTF-8 -*-

"""
collector.aggregation - 聚合数据采集
聚合数据采集是指一次性采集模型分析所需要的数据
====================================================================
"""
import os
import traceback
from tqdm import tqdm
import pandas as pd

import tma
from tma.utils import debug_print
from tma.collector.ts import get_klines
from tma.collector.ts import get_market_basic


# tma.DEBUG = True

def agg_market_klines(k_freq="D", refresh=True, cache=True):
    """获取整个市场全部股票的K线

    :param k_freq: str 默认值 D
        K线周期，可选值参考 `tma.collector.ts.get_klines`
    :param refresh: bool 默认值 True
        是否刷新数据。
        全市场所有股票K线的获取需要较长的时间，默认情况下，获取的数据会
        缓存到用户目录下的`.tma/data`文件夹。当 refresh 为 False 且
        存在对应k_freq的K线数据时，直接读取缓存数据。
    :param cache: bool 默认值 True
        是否缓存数据到用户目录下。
    :return: :class: `pd.DataFrame`
        字段列表:
        ['date', 'open', 'close', 'high', 'low', 'volume', 'code']
    """
    FILE_CACHE = os.path.join(tma.DATA_PATH,
                              "market_klines_%s.csv" % k_freq)
    if os.path.exists(FILE_CACHE) and not refresh:
        df = pd.read_csv(FILE_CACHE, encoding='utf-8', dtype={"code": str})
        return df

    shares = list(get_market_basic().index)
    shares_kls = []
    failed = []
    for share in tqdm(shares):
        try:
            kls = get_klines(share, freq=k_freq)
            shares_kls.append(kls)
        except:
            if tma.DEBUG:
                traceback.print_exc()
            failed.append(share)
    if tma.DEBUG:
        msg = "agg_market_klines(k_freq='%s') 运行结果：总共有%i只股票，" \
              "其中未获取到k线的股票数量是%i" % (
                  k_freq, len(shares), len(failed)
              )
        debug_print(msg, level='INFO')
    df = pd.concat(shares_kls, ignore_index=True)
    if cache:
        # 缓存数据
        df.to_csv(FILE_CACHE, index=False, encoding='utf-8')
    return df



# -*- coding: UTF-8 -*-

import os
import time
import tushare as ts
import pandas as pd
from datetime import datetime

from tma import DATA_PATH

# A股交易日历
# --------------------------------------------------------------------

FILE_CALENDAR = os.path.join(DATA_PATH, 'calendar.csv')
if os.path.exists(FILE_CALENDAR) and \
        time.time() - os.path.getmtime(FILE_CALENDAR) < 3600 * 24:
    trade_calendar = pd.read_csv(FILE_CALENDAR, encoding='utf-8')
else:
    trade_calendar = ts.trade_cal()  # tushare提供的交易日历
    trade_calendar.to_csv(FILE_CALENDAR, index=False, encoding='utf-8')


def is_trade_day(date):
    """判断date日期是不是交易日

    :param date: str or datetime.date, 如 2018-03-15
    :return: Bool
    """
    trade_day = trade_calendar[trade_calendar["isOpen"] == 1]
    trade_day_list = list(trade_day['calendarDate'])
    if isinstance(date, datetime):
        date = str(date.date())
    if date in trade_day_list:
        return True
    else:
        return False


def is_in_trade_time():
    """判断当前是否是交易时间"""
    today = str(datetime.now().date())
    if not is_trade_day(today):  # 如果今天不是交易日，返回False
        return False
    t1 = today + " " + "09:30:00"
    t1 = datetime.strptime(t1, "%Y-%m-%d %H:%M:%S")
    t2 = today + " " + "11:30:00"
    t2 = datetime.strptime(t2, "%Y-%m-%d %H:%M:%S")
    t3 = today + " " + "13:00:00"
    t3 = datetime.strptime(t3, "%Y-%m-%d %H:%M:%S")
    t4 = today + " " + "15:00:00"
    t4 = datetime.strptime(t4, "%Y-%m-%d %H:%M:%S")
    if t1 <= datetime.now() <= t2 or t3 <= datetime.now() <= t4:
        return True
    else:
        return False


def get_recent_trade_days(date, n=10):
    """返回任一日期date前后的n个交易日日期

    :param n: int
        返回交易日的数量n。
        如果 n > 0，则返回date日期未来的n个交易日日期；
        如果 n < 0，则返回date日期过去的n个交易日日期。
    :param date: str  默认值 today
    :return: list
        n个交易日日期列表
    """
    cal_date = list(trade_calendar['calendarDate'])
    date_index = cal_date.index(date)
    if n >= 0:
        recent_days = cal_date[date_index:date_index + n + 20]
    else:
        recent_days = cal_date[date_index + n - 20:date_index + 1]
        recent_days = recent_days[::-1]

    res = []
    for d in recent_days:
        if is_trade_day(d):
            res.append(d)
        if len(res) == abs(n):
            break
    return res


class Calendar:
    """A股交易日历"""

    def __init__(self):
        self.calendar = trade_calendar

    @staticmethod
    def is_trade_time():
        return is_in_trade_time()

    @staticmethod
    def is_trade_day(date):
        return is_trade_day(date)

    @staticmethod
    def recent_trade_days(n, date=None):
        if date is None:
            date = datetime.now().date().__str__()
        return get_recent_trade_days(date=date, n=n)


# --------------------------------------------------------------------

def debug_print(msg, level="INFO"):
    from tma import DEBUG, logger
    LOGS = {
        "INFO": logger.info,
        "DEBUG": logger.debug,
        "EXCEPTION": logger.exception
    }
    if DEBUG:
        LOGS[level](msg)



# -*- coding: UTF-8 -*-

"""
collector.sse - 采集上海证券交易所的数据

官网：http://www.sse.com.cn/
====================================================================
"""

import requests
import pandas as pd


def get_sh_indexes():
    """获取上海证券交易所所有指数的实时行情"""
    url = "http://www.sse.com.cn/js/common/indexQuotes.js"
    res = requests.get(url).text
    lines = res.split("\n")
    lines = [x.replace('_t.push(', "").strip(");'") for x in lines if "_t.push(" in x]
    lines = [
        eval(line, type('Dummy', (dict,), dict(__getitem__=lambda s, n: n))())
        for line in lines
    ]

    index_sh = pd.DataFrame(lines)
    index_sh = index_sh[['JC', 'ZSDM', 'abbr', 'ZRSP', 'DRKP', 'DRSP', 'DRZD', 'DRZX', 'ZDF']]
    index_sh = index_sh.rename(columns={
        'JC': 'name',
        'ZSDM': 'code',
        'abbr': 'kind',
        'ZRSP': 'preclose',
        'DRKP': 'open',
        'DRSP': 'close',
        'DRZD': 'high',
        'DRZX': 'low',
        'ZDF': 'change'
    })

    # index_sh.astype()
    return index_sh



# -*- coding: UTF-8 -*-
import os

# 元信息
# --------------------------------------------------------------------
__name__ = 'tma'
__version__ = "0.1.1"
__author__ = "zengbin"


# 设置用户文件夹
# --------------------------------------------------------------------

def set_path(path=None):
    if not path:
        USER_HOME = os.path.expanduser('~')
        PATH = os.path.join(USER_HOME, ".tma")
    else:
        PATH = path
    DATA_PATH = os.path.join(PATH, "data")
    ACCOUNT_PATH = os.path.join(PATH, "account")
    POOL_PATH = os.path.join(PATH, "pool")
    for P in [PATH, DATA_PATH, ACCOUNT_PATH, POOL_PATH]:
        if not os.path.exists(P):
            os.mkdir(P)
    return PATH, DATA_PATH, ACCOUNT_PATH, POOL_PATH


PATH, DATA_PATH, ACCOUNT_PATH, POOL_PATH = set_path()

# 基本参数配置
# --------------------------------------------------------------------
SCKEY = None
EMAIL = {"user": None, "password": None, "service": None}
DEBUG = False

# 全局日志记录器
from zb.tools.logger import create_logger

logger = create_logger(name='tma', log_file=os.path.join(PATH, "tma.log"), cmd=True)

# API - 列表
# --------------------------------------------------------------------
from .pool import StockPool
from .account import Account
from .rules import RULES

from .utils import Calendar
from .utils import (trade_calendar, is_in_trade_time,
                    is_trade_day, get_recent_trade_days)

# module介绍
# --------------------------------------------------------------------

__doc__ = """
TMA - Tools for Market A - A股工具集
"""



# -*- coding: UTF-8 -*-

"""
collector.ths - 采集同花顺数据

官网：http://www.10jqka.com.cn
数据中心：http://data.10jqka.com.cn/
====================================================================
"""

import requests
from bs4 import BeautifulSoup


def get_market_level():
    url = "http://q.10jqka.com.cn/"
    headers = {
        'User-Agent': 'Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/527+ '
                      '(KHTML, like Gecko, Safari/419.3) Arora/0.6'
    }
    html = requests.get(url, headers=headers)
    html = BeautifulSoup(html.text, 'lxml')
    rate = html.find('div', {'class': "hcharts-right"})
    res = html.find('p', {'id': "hcharts-right"})
    pass
